<?php

/**
 * @file
 * Contains \Drupal\range\Plugin\field\formatter\DefaultRangeFormatter.
 */

namespace Drupal\range\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Parent plugin for decimal and integer range formatters
 */
abstract class DefaultRangeFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, array &$form_state) {
    $elements['range_separator'] = array(
      '#type' => 'textfield',
      '#title' => t('Range separator.'),
      '#default_value' => $this->getSetting('range_separator'),
      '#weight' => 0,
    );
    $options = array(
      '' => t('- None -'),
      '.' => t('Decimal point'),
      ',' => t('Comma'),
      ' ' => t('Space'),
      chr(8201) => t('Thin space'),
      "'" => t('Apostrophe'),
    );
    $elements['thousand_separator'] = array(
      '#type' => 'select',
      '#title' => t('Thousand marker'),
      '#options' => $options,
      '#default_value' => $this->getSetting('thousand_separator'),
      '#weight' => 1,
    );
    $elements['range_combine'] = array(
      '#type' => 'checkbox',
      '#title' => t('Combine equivalent values'),
      '#description' => t('If the FROM and TO values are equal, combine the display into a single value.'),
      '#return_value' => TRUE,
      '#default_value' => $this->getSetting('range_combine'),
      '#weight' => 10,
    );
    $elements['from_prefix_suffix'] = array(
      '#type' => 'checkbox',
      '#title' => t('Display <em>FROM value</em> prefix and suffix.'),
      '#return_value' => TRUE,
      '#default_value' => $this->getSetting('from_prefix_suffix'),
      '#weight' => 11,
    );
    $elements['to_prefix_suffix'] = array(
      '#type' => 'checkbox',
      '#title' => t('Display <em>TO value</em> prefix and suffix.'),
      '#return_value' => TRUE,
      '#default_value' => $this->getSetting('to_prefix_suffix'),
      '#weight' => 12,
    );

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = array();
    $from_value = $this->numberFormat(1234.1234567890);
    $to_value = $this->numberFormat(4321.0987654321);

    $summary[] = $from_value . $this->getSetting('range_separator') . $to_value;
    if ($this->getSetting('range_combine')) {
      $summary[] = t('Equivalent values will be combined into a single value.');
    }
    if ($this->getSetting('from_prefix_suffix')) {
      $summary[] = t('Display with <em>FROM value</em> prefix and suffix.');
    }
    if ($this->getSetting('to_prefix_suffix')) {
      $summary[] = t('Display with <em>TO value</em> prefix and suffix.');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items) {
    $elements = array();

    $combine_range = $this->getSetting('range_combine');

    foreach ($items as $delta => $item) {
      $from_value = $this->numberFormat($item->from);
      $to_value = $this->numberFormat($item->to);

      // Combine values if they are equal.
      if ($combine_range && $from_value === $to_value) {
        $output = $this->viewElementCombined($from_value);
      }
      else {
        $output = $this->viewElementSeparate($from_value, $to_value);
      }

      $elements[$delta] = array('#markup' => $output);
    }

    return $elements;
  }

  /**
   * Helper method. Decides what prefix/suffix use for combined value and
   * returns field markup.
   *
   * FROM and TO might have different prefixes/suffixes.
   * Code below decides which one to use, based on the following:
   *   1. If both are disabled - show naked value
   *   2. If either FROM or TO are enabled - show prefix/suffix of the
   *      enabled one
   *   3. If both are enabled, show prefix from FROM and suffix from TO
   *
   * @param string $value
   *  Field value.
   * @return string
   *  Field markup.
   */
  protected function viewElementCombined($value) {
    $settings = $this->getSettings();
    $field_settings = $this->getFieldSettings();

    $output = '';

    $from_prefix_suffix = !empty($settings['from_prefix_suffix']);
    $to_prefix_suffix = !empty($settings['to_prefix_suffix']);

    // Option #1: both are disabled.
    if (empty($from_prefix_suffix) && empty($to_prefix_suffix)) {
      $output = $value;
    }
    // Option #2a: FROM is enabled.
    elseif (!empty($from_prefix_suffix) && empty($to_prefix_suffix)) {
      $output = $this->viewElementPrefixSuffix($value, TRUE, $field_settings['from']);
    }
    // Option #2b: TO is enabled.
    elseif (empty($from_prefix_suffix) && !empty($to_prefix_suffix)) {
      $output = $this->viewElementPrefixSuffix($value, TRUE, $field_settings['to']);
    }
    // Option #3: both are enabled.
    else {
      $prefix = isset($field_settings['from']['prefix']) ? field_filter_xss($field_settings['from']['prefix']) : '';
      $suffix = isset($field_settings['to']['suffix']) ? field_filter_xss($field_settings['to']['suffix']) : '';
      return $prefix . $value . $suffix;
    }

    return $output;
  }

  /**
   * Helper method. Adds prefixes/suffixes for the field value and
   * returns field markup.
   *
   * @param string $from_value
   *  Field FROM value.
   * @param string $to_value
   *  Field TO value.
   * @return string
   *  Field markup.
   */
  protected function viewElementSeparate($from_value, $to_value) {
    $settings = $this->getSettings();
    $field_settings = $this->getFieldSettings();

    $from = $this->viewElementPrefixSuffix($from_value, $settings['from_prefix_suffix'], $field_settings['from']);
    $to = $this->viewElementPrefixSuffix($to_value, $settings['to_prefix_suffix'], $field_settings['to']);

    return $from . $settings['range_separator'] . $to;
  }

  /**
   * Helper method. Adds prefix and suffix to the given range field value.
   *
   * @param string $value
   *  FROM/TO range field value.
   * @param boolean $display_prefix
   *  Whether to add suffix/prefix or not.
   * @param array $settings
   *  Field instance FROM/TO value settings.
   * @return string
   *  Range field value with added prefix/suffix.
   */
  protected function viewElementPrefixSuffix($value, $display_prefix, $settings) {
    if ($display_prefix) {
      $prefix = isset($settings['prefix']) ? field_filter_xss($settings['prefix']) : '';
      $suffix = isset($settings['suffix']) ? field_filter_xss($settings['suffix']) : '';
      return $prefix . $value . $suffix;
    }

    return $value;
  }

  /**
   * Formats a number.
   *
   * @param mixed $number
   *   The numeric value.
   *
   * @return string
   *   The formatted number.
   */
  abstract protected function numberFormat($number);
}
