<?php

/**
 * @file
 * Contains \Drupal\range\Plugin\Field\FieldType\RangeDecimalItem.
 */

namespace Drupal\range\Plugin\Field\FieldType;

use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Component\Utility\MapArray;

/**
 * Plugin implementation of the 'range_decimal' field type.
 *
 * @FieldType(
 *   id = "range_decimal",
 *   label = @Translation("Range (decimal)"),
 *   description = @Translation("This field stores a fixed decimal range in the database."),
 *   settings = {
 *     "precision" = "10",
 *     "scale" = "2"
 *   },
 *   instance_settings = {
 *     "min" = "",
 *     "max" = "",
 *     "from" = {
 *       "prefix" = "",
 *       "suffix" = ""
 *     },
 *     "to" = {
 *       "prefix" = "",
 *       "suffix" = ""
 *     }
 *   },
 *   default_widget = "range",
 *   default_formatter = "range_decimal"
 * )
 */
class RangeDecimalItem extends RangeItemBase {

  /**
   * {@inheritdoc}
   */
  public function getPropertyDefinitions() {
    if (!isset(static::$propertyDefinitions)) {
      static::$propertyDefinitions['from'] = DataDefinition::create('string')
        ->setLabel(t('From value'));
      static::$propertyDefinitions['to'] = DataDefinition::create('string')
        ->setLabel(t('To value'));
    }
    return static::$propertyDefinitions;
  }

  /**
   * {@inheritdoc}
   */
  public function instanceSettingsForm(array $form, array &$form_state) {
    $field_settings = $this->definition->getSettings();

    $element = parent::instanceSettingsForm($form, $form_state);

    $step = pow(0.1, $field_settings['scale']);
    $element['min']['#step'] = $element['max']['#step'] = $step;

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldDefinitionInterface $field_definition) {
    return array(
      'columns' => array(
        'from' => array(
          'type' => 'numeric',
          'precision' => $field_definition->settings['precision'],
          'scale' => $field_definition->settings['scale'],
          'not null' => FALSE,
        ),
        'to' => array(
          'type' => 'numeric',
          'precision' => $field_definition->settings['precision'],
          'scale' => $field_definition->settings['scale'],
          'not null' => FALSE,
        ),
      ),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, array &$form_state, $has_data) {
    $element = array();
    $settings = $this->getSettings();

    $element['precision'] = array(
      '#type' => 'select',
      '#title' => t('Precision'),
      '#options' => MapArray::copyValuesToKeys(range(10, 32)),
      '#default_value' => $settings['precision'],
      '#description' => t('The total number of digits to store in the database, including those to the right of the decimal.'),
      '#disabled' => $has_data,
    );
    $element['scale'] = array(
      '#type' => 'select',
      '#title' => t('Scale', array(), array('decimal places')),
      '#options' => MapArray::copyValuesToKeys(range(0, 10)),
      '#default_value' => $settings['scale'],
      '#description' => t('The number of digits to the right of the decimal.'),
      '#disabled' => $has_data,
    );

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave() {
    $this->from = round($this->from, $this->getSetting('scale'));
    $this->to = round($this->to, $this->getSetting('scale'));
  }

}
