<?php

/**
 * @file
 * Contains \Drupal\range\Plugin\Field\FieldWidget\RangeWidget.
 */

namespace Drupal\range\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Symfony\Component\Validator\ConstraintViolationInterface;

/**
 * Plugin implementation of the 'range' widget.
 *
 * @FieldWidget(
 *   id = "range",
 *   label = @Translation("Text fields"),
 *   field_types = {
 *     "range_integer",
 *     "range_float",
 *     "range_decimal"
 *   }
 * )
 */
class RangeWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, array &$form_state) {
    $from = isset($items[$delta]->from) ? $items[$delta]->from : NULL;
    $to = isset($items[$delta]->to) ? $items[$delta]->to : NULL;
    $field_settings = $this->getFieldSettings();

    // Wrap in a fieldset for single field.
    if ($this->fieldDefinition->getCardinality() === 1) {
      $element['#type'] = 'fieldset';
    }

    $base = array(
      '#type' => 'number',
      '#required' => $element['#required'],
    );

    // Set the step for floating point and decimal numbers.
    switch ($this->fieldDefinition->getType()) {
      case 'range_decimal':
        $base['#step'] = pow(0.1, $field_settings['scale']);
        break;

      case 'range_float':
        $base['#step'] = 'any';
        break;
    }

    // Set minimum and maximum.
    if (is_numeric($field_settings['min'])) {
      $base['#min'] = $field_settings['min'];
    }
    if (is_numeric($field_settings['max'])) {
      $base['#max'] = $field_settings['max'];
    }

    $element['from'] = array(
      '#title' => t('From'),
      '#default_value' => $from,
      '#prefix' => '<div class="field-widget-range-subelements-wrapper clearfix">',
    ) + $base;

    $element['to'] = array(
      '#title' => t('to'),
      '#default_value' => $to,
      '#suffix' => '</div>',
    ) + $base;

    // Add prefixes and suffixes.
    $this->formElementSubElementPrefixSuffix($element, 'from');
    $this->formElementSubElementPrefixSuffix($element, 'to');

    $element['#attached']['css'][] = drupal_get_path('module', 'range') . '/css/range.admin.css';

    return $element;
  }

  /**
   * Helper method. Adds prefix and suffix to a range field widget FROM/TO
   * subelements.
   *
   * @param array $element
   *  Range field widget definition array.
   * @param string $element_name
   *  Form element machine name.
   */
  protected function formElementSubElementPrefixSuffix(array &$element, $element_name) {
    $setting = $this->getFieldSetting($element_name);
    if (!empty($setting['prefix'])) {
      $prefix = field_filter_xss($setting['prefix']);
      $element[$element_name]['#field_prefix'] = $prefix;
    }
    if (!empty($setting['suffix'])) {
      $suffix = field_filter_xss($setting['suffix']);
      $element[$element_name]['#field_suffix'] = $suffix;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function errorElement(array $element, ConstraintViolationInterface $error, array $form, array &$form_state) {
    return $element;
  }

}
